//
//  RTMManager.m
//  IntregratingRTMTutorial
//
//  Created by Michael Keith on 30/12/11.
//  Copyright (c) 2011 Ziggurat. All rights reserved.
//

#import "RTMManager.h"
#include "rtmapi.h"
#import "RXMLElement.h"
#import "ISO8601DateFormatter.h"
#import "Task.h"
#import "TaskSeries.h"
#import "TaskManager.h"

#define MY_API_KEY "the API key you obtained from RTM"
#define MY_API_SECRET "the API secret you obtained from RTM"

static RTMManager *_sharedManager = nil;

@implementation RTMManager {
	rtm_api *rtm;
    NSString *frob;
    char *authToken;
    ISO8601DateFormatter *dateFormatter;

}

+ (RTMManager *)sharedRTMManager {
    if (!_sharedManager)
        _sharedManager = [[RTMManager alloc] init];
    return _sharedManager;
}

+ (void)cleanUp {
    _sharedManager = nil;
}

- (id)init {
    self = [super init];
    if (self) {
        rtm = rtm_api_init(MY_API_KEY, MY_API_SECRET);
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(authenticateComplete) name:@"authenticatecomplete" object:nil];

    }
    return self;
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    
    rtm_api_free(rtm);   
    rtm = NULL;
    
    free(authToken);
    authToken = NULL;
}

- (BOOL)checkResponse:(RXMLElement *)responseElement {
    if (!responseElement)
        return NO;
    
    if (![responseElement.tag isEqualToString:@"rsp"]) {
        NSLog(@"Non rsp response: %@", responseElement.tag);
        return YES;
    }
    
    if ([[responseElement attribute:@"stat"] isEqualToString:@"ok"]) {
        return YES;
    }
    
    RXMLElement *errElement = [responseElement child:@"err"];
    NSString *errCode = [errElement attribute:@"code"];
    NSString *errMsg = [errElement attribute:@"msg"];
        
    NSLog(@"Failed with code '%@' and message '%@'", errCode, errMsg);
    
    return NO;
}

- (void)authenticate {
    char *response = rtm_api_call_method(rtm, "rtm.auth.getFrob"); // 1
        
    NSString *responseString = [NSString stringWithCString:response encoding:NSUTF8StringEncoding];
        
    RXMLElement *rxml = [RXMLElement elementFromXMLString:responseString]; // 2
    if (![self checkResponse:rxml]) { // 3
        NSLog(@"getFrob failed");
        return;
    }
    
    RXMLElement *frobElement = [rxml child:@"frob"]; // 4
    frob= frobElement.text;            
    NSLog(@"got frob: %@", frob);
                            
    char *url = rtm_api_auth_get_url(rtm, "read", [frob cStringUsingEncoding:NSUTF8StringEncoding]); // 5
    NSLog(@"Authentication URL: %s\n", url);    

    NSString *authURL = [NSString stringWithCString:url encoding:NSUTF8StringEncoding];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"showauthenticatewindow" object:authURL]; // 6
}

- (NSDate *)dateFromString:(NSString *)dateString {
    if ([dateString length] == 0)
        return nil;
    
    if (!dateFormatter) dateFormatter = [[ISO8601DateFormatter alloc] init];
    return [dateFormatter dateFromString:dateString];
}

- (void)setTaskSeries:(TaskSeries *)series fromElement:(RXMLElement *)element {
    series.taskSeriesId = [element attribute:@"id"];
    series.name = [element attribute:@"name"];
}

- (void)setTask:(Task *)task fromElement:(RXMLElement *)element {
    task.taskId = [element attribute:@"id"];
    task.dateDue = [self dateFromString:[element attribute:@"due"]];                
    task.hasDueTime = ![[element attribute:@"has_due_time"] isEqualToString:@"0"];
}

- (void)getTasksFromXML:(RXMLElement *)rxml {
    RXMLElement *rootTasks = [rxml child:@"tasks"];
    NSArray *lists = [rootTasks children:@"list"];
    
    if (lists == nil) {
        lists = [rxml children:@"list"];
    }
    
    for (RXMLElement *list in lists) {
        NSArray *taskSeriesList = [list children:@"taskseries"];
        for (RXMLElement *taskSeries in taskSeriesList) {
            TaskSeries *newTaskSeries = [[TaskSeries alloc] init];
            
            [self setTaskSeries:newTaskSeries fromElement:taskSeries];
            
            NSArray *tasks = [taskSeries children:@"task"];
            for (RXMLElement *taskElement in tasks) {
                Task *newTask = newTask = [[Task alloc] init];
                [self setTask:newTask fromElement:taskElement];
                
                newTask.ownerTaskSeries = newTaskSeries;
                [newTaskSeries addTask:newTask];
            }
            [[TaskManager sharedTaskManager] addTaskSeries:newTaskSeries];
        }
    }
    
   [[NSNotificationCenter defaultCenter] postNotificationName:@"tasklistupdatednotification" object:nil];
}

- (void)getTaskList {
    rtm_api_set_auth_token(rtm, authToken);
    rtm_api_param_add(rtm, "filter", "status:incomplete");
        
    char *response = rtm_api_call_authenticated_method(rtm, "rtm.tasks.getList");
    NSString *responseString = [NSString stringWithCString:response encoding:NSUTF8StringEncoding];

    RXMLElement *rxml = [RXMLElement elementFromXMLString:responseString];
    if (![self checkResponse:rxml]) {
        NSLog(@"rtm.tasks.getList failed");
        return;
    }

    [self getTasksFromXML:rxml];
}

- (void)authTokenObtained {
    [self getTaskList];       
}

- (void)authenticateComplete {
    rtm_api_param_clear(rtm);
    rtm_api_param_add(rtm, "frob", [frob cStringUsingEncoding:NSUTF8StringEncoding]);
    char *response = rtm_api_call_method(rtm, "rtm.auth.getToken");
        
    NSString *responseString = [NSString stringWithCString:response encoding:NSUTF8StringEncoding];
    NSLog(@"getToken response: %@", responseString);
    RXMLElement *rxml = [RXMLElement elementFromXMLString:responseString];
    if (![self checkResponse:rxml]) {
        // we failed
        return;
    }
            
    RXMLElement *tokenElement = [rxml child:@"auth.token"];                        

    NSString *authTokenString = [tokenElement.text copy];
            
    authToken = malloc(([authTokenString length] + 1) * sizeof(char));
    strncpy(authToken, [authTokenString cStringUsingEncoding:NSUTF8StringEncoding], [authTokenString length]);
    authToken[[authTokenString length]] = '\0';                
                
    [self authTokenObtained];                                        
}

@end
